from os import urandom, getenv

from flask import Flask, render_template, request, session, url_for

from model import db_session, init_db, User, PasswordHasher

BASEURL = getenv('BASEURL', '')
init_db()
app = Flask(__name__)
app.secret_key = urandom(32)


def get_url(endpoint):
    return f'{BASEURL}{url_for(endpoint)}'
app.jinja_env.globals.update(get_url=get_url)


@app.teardown_appcontext
def remove_db_session(exception=None):
    db_session.remove()


@app.route('/', methods=['GET', 'POST'])
def index():
    if request.method == 'POST':
        user = User.query.filter(User.username == request.form['username']).first()

        if not user or not PasswordHasher.verify(request.form['password'], user.passwordhash):
            return render_template('login.html', alert='Invalid credentials!')
        else:
            session['logged_in'] = True
            session['username'] = request.form['username']
            return render_template('internal.html')
    
    if session.get('logged_in'):
        return render_template('internal.html')
    return render_template('login.html')


@app.route('/register', methods=['GET', 'POST'])
def register():
    if request.method == 'POST':
        validate_register_fields(request)

        if User.query.filter(User.username == request.form['username']).all():
            return render_template('register.html', alert='Username already in use.')

        db_session().add(User(username=request.form['username'],
                              passwordhash=PasswordHasher.hash(request.form['password'])))
        db_session().commit()

        return render_template('login.html', success='Account "{}" successfully registered. You can log in now!'.format(request.form['username']))

    return render_template('register.html')


def validate_register_fields(request):
    if not request.form['username'] or not request.form['password'] or not request.form['passwordconfirm']:
        return render_template('register.html', alert='You need to fill everything.')
    if request.form['password'] != request.form['passwordconfirm']:
        return render_template('register.html', alert='Passwords do not match! Please try again.')


@app.route('/logout')
def logout():
    try:
        session.pop('logged_in')
        session.pop('username')
    except KeyError:
        pass
    return render_template('login.html')


@app.errorhandler(401)
@app.errorhandler(404)
@app.route('/error')
def error(error):
    return render_template('error.html', error=error), error.code


# 500 needs a separate handler, as Flask would print
# the actual piece of code that caused the exception
# for some bizarre reason
@app.errorhandler(500)
@app.route('/error')
def servererror(error):
    return render_template('error.html', error='500: Internal server error'), 500


if __name__ == '__main__':
    app.run(host='127.0.0.1', debug=True, port=11111)
